<?php

declare(strict_types=1);

namespace Drupal\multisite_manager_tool\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\DrupalKernel;
use Symfony\Component\HttpFoundation\Request;


/**
 * Provides a Multisite Manager Tool form.
 */
final class MultisiteManagerToolForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'multisite_manager_tool_multisite_manager_tool';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $sites = $this->getMultisiteList();
    foreach($sites as $site) {
      $rows[]=[
        'site' => $site
      ];
    }


    if (empty($rows)) {
      $this->messenger()->addWarning(
        t('There are no sites activated in your multisite. See the <a href=":multisite_drupal" target="_blank">Multisite Drupal</a> page for more information.', 
        [':multisite_drupal' => 'https://www.drupal.org/docs/getting-started/multisite-drupal'])
      );
      return $form;
    }
  
    $form['table'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Site'),
        $this->t('Actions'),
      ],
      '#rows' => [],
    ];

    

    foreach ($rows as $key => $row) {
      $form['table'][$key]['site'] = [
        '#plain_text' => $row['site'],
      ];
      $form['table'][$key]['actions'] = [
        '#type' => 'actions',
        'edit' => [
          '#type' => 'submit',
          '#value' => $this->t('Clear cache'),
          '#name' => "clear_{$row['site']}",
          '#submit' => ['::clearCacheSite'],
          '#limit_validation_errors' => [],
          '#attributes' => ['class' => ['button', 'button--primary']],
        ],
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    
  }

  /**
   * Submit handler for the "Clear cache" button.
   */
  public function clearCacheSite(array &$form, FormStateInterface $form_state) {
    $trigger = $form_state->getTriggeringElement();
    preg_match('/clear_(.+)/', $trigger['#name'], $matches);
    
    if (isset($matches[1])) {
      $site = $matches[1];
      $site_path = DRUPAL_ROOT . "/sites/$site";

      if (!is_dir($site_path)) {
        \Drupal::messenger()->addError($this->t('Site: @site not found',['@site' => $site]));
        return;
      }

      $command = "drush --root=" . escapeshellarg(DRUPAL_ROOT) . " --uri=" . escapeshellarg($site) . " cache:rebuild";
      exec($command, $output, $return_var);
      \Drupal::messenger()->addMessage($this->t('Caches cleared in: @site', ['@site' => $site]));
    }
  }

  /**
   * Get the list of available multisites.
   */
  private function getMultisiteList():array {
    $sites_directory = DRUPAL_ROOT . '/sites/';
    $sites = [];

    foreach (scandir($sites_directory) as $folder) {
      if ($folder === '.' || $folder === '..' || in_array($folder, ['default', 'all'])) {
        continue; // Ignorar carpetas irrelevantes
      }
      if (is_dir("$sites_directory$folder") && file_exists("$sites_directory$folder/settings.php")) {
        $sites[$folder] = $folder;
      }
    }

    return $sites;
  }

}
